Module cast

Module cast 

Type casting between scalar formats.

This module provides:

  • CastDtype: Trait marking types eligible for bulk casting
  • cast: Bulk-converts a slice from one scalar format to another

Traits

CastDtype
Trait for types that can participate in cast operations.

Functions

cast
Cast source slice elements to destination slice.