Module cast
numkong
7.2.4
Module cast
Module Items
Traits
Functions
In crate numkong
numkong
Module
cast
Type casting between scalar formats.
This module provides:
CastDtype
: Trait marking types eligible for bulk casting
cast
: Bulk-converts a slice from one scalar format to another
Traits

Cast
Dtype
Trait for types that can participate in cast operations.
Functions

cast
Cast source slice elements to destination slice.